@@ -10,9 +10,12 @@ from .. import utils
 __all__ = [
     "get_data_list",
     "get_time_values",
+    "get_vector_time_values",
     "get_batch_values",
+    "get_vector_batch_values",
     "get_multiple_data_values",
     "insert_time_values",
+    "insert_vector_time_values",
 ]
 
 
@@ -61,13 +64,13 @@ def get_data_list(
     multithread_pages: bool, default False
         Only used when getting all pages. If True, pages are retrieved in multiple threads simultaneously.
     expand_measurement: bool, default True
-        Whether or not measurement information should be expanded into multiple columns.
+        Whether the measurement information should be expanded into multiple columns.
     extract_from_storage_unit: {'id', 'label', None}, default 'label'
         What field should be extracted from storage unit information. If None, the full dictionary is kept.
     extract_from_tags: {'id', 'value', None}, default 'value'
         What field should be extracted for naming tags. If None, the full dictionary is kept.
     expand_tags: bool, default True
-        Whether or not tags should be expanded into multiple columns.
+        Whether tags should be expanded into multiple columns.
     api_credentials: OIAnalyticsAPICredentials, optional
         The credentials to use to query the API. If None, previously set default credentials are used.
 
@@ -274,6 +277,149 @@ def get_time_values(
     return ser
 
 
+def get_vector_time_values(
+    vector_data_id: str,
+    start_date: Union[str, datetime],
+    end_date: Union[str, datetime],
+    aggregation: str = "RAW_VALUES",
+    index_aggregation: str = "RAW_VALUES",
+    aggregation_period: Optional[Union[str, timedelta, pd.Timedelta]] = None,
+    max_number_of_points: Optional[int] = None,
+    aggregation_function: Optional[str] = None,
+    index_aggregation_function: Optional[str] = None,
+    index_aggregation_step_type: Optional[str] = None,
+    index_aggregation_step_value: Optional[float] = None,
+    min_index: Optional[float] = None,
+    max_index: Optional[float] = None,
+    index_unit_id: Optional[str] = None,
+    value_unit_id: Optional[str] = None,
+    api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
+) -> Union[pd.DataFrame, pd.Series]:
+    """
+    Get values from a vector temporal data from the OIAnalytics API
+
+    Parameters
+    ----------
+    vector_data_id: string
+        The data id on which values are to be retrieved
+    start_date: datetime or string
+        The beginning of the period to be retrieved
+    end_date: datetime or string
+        The end of the period to be retrieved
+    index_unit_id: string, optional
+        The id of the unit to express the index data set. If not present a default unit will be used.
+        This unit should be compatible with the physical quantity of the data's index queried.
+    value_unit_id: string, optional
+        The id of the unit to express the values. If not presented, a default unit will be used.
+        This unit should be compatible with the physical quantity of the data queried.
+    aggregation: string {'RAW_VALUES', 'TIME', 'GLOBAL'}
+        How to aggregate the values by time. If 'TIME', aggregation period and function should be specified.
+    index_aggregation: string {'RAW_VALUES', 'RESAMPLE', 'GLOBAL'}
+        How to aggregate the values by index.
+        If 'RESAMPLE', index aggregation function, step type and step value should be specified.
+        If 'GLOBAL', index aggregation function should be specified.
+    aggregation_function: string, optional
+        Required in case 'aggregation' is 'TIME' and is the aggregation function to use to aggregate the values within
+        the sampling period.
+        Possible values are: 'FIRST', 'LAST', 'LAST_MINUS_FIRST', 'SUM', 'MIN', 'MAX', 'MEAN', 'MEDIAN', 'STDEV',
+        'PERCENTILE5', 'PERCENTILE95', 'DECILE1', 'DECILE9', 'QUARTILE1', 'QUARTILE9', 'COUNT', 'MEAN_MINUS_SIGMA',
+        'MEAN_PLUS_SIGMA', 'MEAN_MINUS_TWO_SIGMA', 'MEAN_PLUS_TWO_SIGMA', 'MEAN_MINUS_THREE_SIGMA',
+        'MEAN_PLUS_THREE_SIGMA', 'VALUE_CHANGE', 'UNIQUE', 'WEIGHTED_AVERAGE'.
+    aggregation_period: string, optional
+        Required in case 'aggregation' is 'TIME' and is the sampling period of the expected result.
+        This period should be an ISO period as described in https://en.wikipedia.org/wiki/ISO_8601#Durations
+    max_number_of_points: integer, optional
+        The number of time intervals between 'start_date' and 'end_date'
+    index_aggregation_function: string, optional
+        Defines the index aggregation function
+    index_aggregation_step_type: string, optional
+        Method to define the interval for index aggregation; acceptable values: {"INTERVAL", "NUMBER_OF_POINTS"}.
+    index_aggregation_step_value: float or int, optional
+        Or index interval or number of points used for index aggregation, according to the choice of 'index_aggregation_step_type'.
+    min_index: float, optional
+        The minimum index value to be presented.
+    max_index: float, optional
+        The maximum index value to be presented.
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+    A Series containing the values of the data, indexed by 'timestamps'
+    """
+
+    # Query endpoint
+    data = endpoints.data.get_vector_time_values(
+        vector_data_id=vector_data_id,
+        start_date=start_date,
+        end_date=end_date,
+        index_unit_id=index_unit_id,
+        value_unit_id=value_unit_id,
+        aggregation=aggregation,
+        index_aggregation=index_aggregation,
+        aggregation_period=aggregation_period,
+        max_number_of_points=max_number_of_points,
+        aggregation_function=aggregation_function,
+        index_aggregation_function=index_aggregation_function,
+        index_aggregation_step_type=index_aggregation_step_type,
+        index_aggregation_step_value=index_aggregation_step_value,
+        min_index=min_index,
+        max_index=max_index,
+        api_credentials=api_credentials,
+    )
+
+    if data["type"] == "vector-time-values":
+        df = pd.DataFrame(
+            columns=data["indexes"],
+            data=data["values"],
+            dtype=float if len(data["values"]) == 0 else None,
+        )
+    elif data["type"] == "vector-time-range-values":
+        dfs_list = []
+        keys = []
+        for key in ["lows", "values", "highs"]:
+            if len(data[key]) > 0:
+                dfs_list.append(
+                    pd.DataFrame(
+                        columns=data["indexes"],
+                        data=data[key],
+                        dtype=float if len(data[key]) == 0 else None,
+                    )
+                )
+                keys.append(key)
+        df = pd.concat(dfs_list, axis=1, keys=keys)
+    elif data["type"] == "time-values":
+        df = pd.Series(
+            data=data["values"], dtype=float if len(data["values"]) == 0 else None,
+        )
+    elif data["type"] == "time-range-value":
+        series_list = []
+        cols = []
+        for col in ["lows", "values", "highs"]:
+            if len(data[col]) > 0:
+                series_list.append(
+                    pd.Series(
+                        data=data[col],
+                        dtype=float if len(data["values"]) == 0 else None,
+                    )
+                )
+                cols.append(col)
+        if len(cols) == 0:
+            df = pd.DataFrame()
+        else:
+            df = pd.concat(series_list, axis=1, keys=cols)
+
+    if len(data["values"]) == 0:
+        df.index = pd.to_datetime(data["timestamps"]).tz_localize("UTC")
+    else:
+        df.index = pd.to_datetime(data["timestamps"])
+
+    df.index.name = "timestamp"
+
+    # Output
+    return df
+
+
 def get_batch_values(
     data_id: str,
     start_date: Union[str, datetime],
@@ -377,11 +523,7 @@ def get_batch_values(
 
     ser = pd.Series(
         name=data_name,
-        index=[
-            data["batchIds"],
-            data["batchNames"],
-            timestamp_index,
-        ]
+        index=[data["batchIds"], data["batchNames"], timestamp_index]
         if data_type == "batch-values"
         else timestamp_index,
         data=data["values"],
@@ -397,6 +539,168 @@ def get_batch_values(
     return ser
 
 
+def get_vector_batch_values(
+    vector_data_id: str,
+    start_date: Union[str, datetime],
+    end_date: Union[str, datetime],
+    index_unit_id: Optional[str] = None,
+    value_unit_id: Optional[str] = None,
+    aggregation: str = "RAW_VALUES",
+    index_aggregation: str = "RAW_VALUES",
+    aggregation_period: Optional[Union[str, timedelta, pd.Timedelta]] = None,
+    max_number_of_points: Optional[int] = None,
+    aggregation_function: Optional[str] = None,
+    index_aggregation_function: Optional[str] = None,
+    index_aggregation_step_type: Optional[str] = None,
+    index_aggregation_step_value: Optional[float] = None,
+    min_index: Optional[float] = None,
+    max_index: Optional[float] = None,
+    batch_ids: Optional[List[str]] = None,
+    api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
+) -> Union[pd.DataFrame, pd.Series]:
+    """
+    Get values from a vector temporal data from the OIAnalytics API
+
+    Parameters
+    ----------
+    vector_data_id: string
+        The data id on which values are to be retrieved
+    start_date: datetime or string
+        The beginning of the period to be retrieved
+    end_date: datetime or string
+        The end of the period to be retrieved
+    index_unit_id: string
+        The id of the unit to express the index data set. If not present a default unit will be used.
+        This unit should be compatible with the physical quantity of the data's index queried.
+    value_unit_id: string
+        The id of the unit to express the values. If not presented, a default unit will be used.
+        This unit should be compatible with the physical quantity of the data queried.
+    aggregation: string {'RAW_VALUES', 'TIME'}
+        How to aggregate the values by time. If 'TIME', aggregation period and function should be specified.
+    index_aggregation: string, optional {'RAW_VALUES', 'RESAMPLE', 'GLOBAL'}
+        How to aggregate the values by index.
+        If 'RESAMPLE', index aggregation function, step type and step value should be specified.
+        If 'GLOBAL', index aggregation function should be specified.
+    aggregation_function: string, optional
+        Required in case 'aggregation' is 'TIME' and is the aggregation function to use to aggregate the values within
+        the sampling period.
+        Possible values are: 'FIRST', 'LAST', 'LAST_MINUS_FIRST', 'SUM', 'MIN', 'MAX', 'MEAN', 'MEDIAN', 'STDEV',
+        'PERCENTILE5', 'PERCENTILE95', 'DECILE1', 'DECILE9', 'QUARTILE1', 'QUARTILE9', 'COUNT', 'MEAN_MINUS_SIGMA',
+        'MEAN_PLUS_SIGMA', 'MEAN_MINUS_TWO_SIGMA', 'MEAN_PLUS_TWO_SIGMA', 'MEAN_MINUS_THREE_SIGMA',
+        'MEAN_PLUS_THREE_SIGMA', 'VALUE_CHANGE', 'UNIQUE', 'WEIGHTED_AVERAGE'.
+    aggregation_period: string, optional
+        Required in case 'aggregation' is 'TIME' and is the sampling period of the expected result.
+        This period should be an ISO period as described in https://en.wikipedia.org/wiki/ISO_8601#Durations
+    max_number_of_points: integer greater or equal to 1
+        Maximum number of points inside the time interval after aggregation
+    index_aggregation_function: string, optional
+        Defines the index aggregation function
+    index_aggregation_step_type: string, optional
+        Method to define the interval for index aggregation; acceptable values: {"INTERVAL", "NUMBER_OF_POINTS"}.
+    index_aggregation_step_value: float or int, optional
+        Or index interval or number of points used for index aggregation, according to the choice of 'index_aggregation_step_type'.
+    min_index: float, optional
+        The minimum index value to be presented.
+    max_index: float, optional
+        The maximum index value to be presented.
+    batch_ids: list of strings
+        A list containing the batch ids
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+    A DataFrame or a Series containing the values of the data, indexed by <'batches-ids' and 'batches-names'> and / or only by <'timestamps'>.
+    """
+
+    # Query endpoint
+    data = endpoints.data.get_vector_batch_values(
+        vector_data_id=vector_data_id,
+        start_date=start_date,
+        end_date=end_date,
+        batch_ids=batch_ids,
+        aggregation=aggregation,
+        index_aggregation=index_aggregation,
+        aggregation_period=aggregation_period,
+        aggregation_function=aggregation_function,
+        max_number_of_points=max_number_of_points,
+        index_aggregation_function=index_aggregation_function,
+        index_aggregation_step_type=index_aggregation_step_type,
+        index_aggregation_step_value=index_aggregation_step_value,
+        min_index=min_index,
+        max_index=max_index,
+        index_unit_id=index_unit_id,
+        value_unit_id=value_unit_id,
+        api_credentials=api_credentials,
+    )
+
+    if len(data["values"]) == 0:
+        timestamp_index = pd.to_datetime(data["timestamps"]).tz_localize("UTC")
+    else:
+        timestamp_index = pd.to_datetime(data["timestamps"])
+
+    if data["type"] in [
+        "batch-vector-range-value",
+        "batch-range-value",
+        "vector-batch-values",
+        "batch-values",
+    ]:
+        df_index = [data["batchIds"], data["batchNames"], timestamp_index]
+        df_index_name = ["batch_id", "batch_name", "timestamp"]
+    elif data["type"] in [
+        "time-vector-range-value",
+        "time-range-value",
+        "vector-time-values",
+        "time-values",
+    ]:
+        df_index = timestamp_index
+        df_index_name = "timestamp"
+
+    if data["type"] in ["vector-batch-values", "vector-time-values"]:
+        df = pd.DataFrame(
+            columns=data["indexes"],
+            data=data["values"],
+            dtype=float if len(data["values"]) == 0 else None,
+        )
+    elif data["type"] in ["time-values", "batch-values"]:
+        df = pd.Series(
+            data=data["values"], dtype=float if len(data["values"]) == 0 else None,
+        )
+    elif data["type"] in ["time-vector-range-value", "batch-vector-range-value"]:
+        dfs_list = []
+        keys = []
+        for key in ["lows", "values", "highs"]:
+            if len(data[key]) > 0:
+                dfs_list.append(
+                    pd.DataFrame(
+                        columns=data["indexes"],
+                        data=data[key],
+                        dtype=float if len(data[key]) == 0 else None,
+                    )
+                )
+                keys.append(key)
+        df = pd.concat(dfs_list, axis=1, keys=keys)
+    elif data["type"] in ["time-range-value", "batch-range-value"]:
+        series_list = []
+        cols = []
+        for col in ["lows", "values", "highs"]:
+            if len(data[col]) > 0:
+                series_list.append(
+                    pd.Series(
+                        data=data[col],
+                        dtype=float if len(data["values"]) == 0 else None,
+                    )
+                )
+                cols.append(col)
+        df = pd.concat(series_list, axis=1, keys=cols)
+
+    df.index = df_index
+    df.index.names = df_index_name
+
+    # Output
+    return df
+
+
 def get_multiple_data_values(
     start_date: Union[str, datetime],
     end_date: Union[str, datetime],
@@ -518,11 +822,7 @@ def get_multiple_data_values(
 
         ser = pd.Series(
             name=data_name,
-            index=[
-                data["batchIds"],
-                data["batchNames"],
-                timestamp_index,
-            ]
+            index=[data["batchIds"], data["batchNames"], timestamp_index,]
             if data_type == "batch-values" and join_series_on != "timestamp"
             else timestamp_index,
             data=data["values"],
@@ -570,7 +870,7 @@ def insert_time_values(
     units: dict, optional
         A dictionary indexed by data reference, specifying the values in which it is sent.
     use_external_reference: bool, default False
-        Whether or not the data are named using their external reference (else the OIAnalytics reference is used).
+        Whether the data are named using their external reference (else the OIAnalytics reference is used).
     timestamp_index_name: str, default 'timestamp'
         Name of the index level containing the timestamp
     api_credentials: OIAnalyticsAPICredentials, optional
@@ -610,11 +910,13 @@ def insert_time_values(
         ser.name = "value"
 
         # Build payload for the data
-        col_dict = {"dataReference": col}
-        col_dict["unit"] = units.get(col, None)
-        col_dict["values"] = (
-            pd.DataFrame(ser).reset_index().dropna().to_dict(orient="records")
-        )
+        col_dict = {
+            "dataReference": col,
+            "unit": units.get(col, None),
+            "values": (
+                pd.DataFrame(ser).reset_index().dropna().to_dict(orient="records")
+            ),
+        }
 
         payload.append(col_dict)
 
@@ -627,3 +929,87 @@ def insert_time_values(
 
     # Output
     return response
+
+
+def insert_vector_time_values(
+    data: Union[pd.DataFrame, List[pd.DataFrame]],
+    data_reference: Union[str, List[str]],
+    index_units: Optional[dict] = None,
+    values_units: Optional[dict] = None,
+    use_external_reference: bool = False,
+    timestamps_index_name: str = "timestamp",
+    api_credentials: Optional[_credentials.OIAnalyticsAPICredentials] = None,
+):
+    """
+    Insert time values stored in a DataFrame through the OIAnalytics API
+
+    Parameters
+    ----------
+    data: dataframe or list of dataframes
+        Element(s) containing the vector data time values to be inserted
+    data_reference: string or list of strings
+        The unique data reference for the data being inserted
+    index_units: dictionary, optional
+        A dictionary indexed by data reference, specifying the values in which it is sent
+    values_units: dictionary, optional
+        A dictionary indexed by data reference, specifying the values in which it is sent
+    use_external_reference: bool, default False
+        Whether the data are named using their external reference (else the OIAnalytics reference is used).
+    timestamps_index_name: str, default 'timestamps'
+        Name of the index level containing the timestamps
+    api_credentials: OIAnalyticsAPICredentials, optional
+        The credentials to use to query the API. If None, previously set default credentials are used.
+
+    Returns
+    -------
+    A dictionary of the response from the API, containing the data insert report
+    """
+
+    # Init 'index_units
+    if index_units is None:
+        index_units = {}
+
+    # Init values_units
+    if values_units is None:
+        values_units = {}
+
+    # Init list of data and list of data_reference
+    if isinstance(data, pd.DataFrame):
+        data = [data.copy()]
+        data_reference = [data_reference]
+
+    # Build DTO
+    payload = []
+    for reference, df in zip(data_reference, data):
+        try:
+            timestamps = (
+                df.index.get_level_values(timestamps_index_name)
+                .to_series()
+                .apply(utils.get_zulu_isoformat)
+                .rename(timestamps_index_name)
+            )
+        except KeyError:
+            raise KeyError(
+                "The dataframe must have an index level named 'timestamp' containing datetime-like values"
+            )
+        # Build payload for the data
+        payload.append(
+            {
+                "dataReference": reference,
+                "indexUnit": index_units.get(reference, None),
+                "valueUnit": values_units.get(reference, None),
+                "timestamps": timestamps.tolist(),
+                "indexes": df.columns.tolist(),
+                "values": df.to_numpy().tolist(),
+            }
+        )
+
+    # Query endpoint
+    response = endpoints.data.insert_vector_time_values(
+        data=payload,
+        use_external_reference=use_external_reference,
+        api_credentials=api_credentials,
+    )
+
+    # Output
+    return response
